@@ -112,7 +112,16 @@ class HttpRequest(object):
         url_parts = list(urlparse(str(url)))
         query = dict(parse_qsl(url_parts[4]))
         query.update(query_args)
-        url_parts[4] = urlencode(query)
+
+        url_parts[4] = urlencode(
+            OrderedDict(
+                pair
+                for pair in sorted(six.iteritems(query),
+                                   key=lambda pair: pair[0])
+            ),
+            doseq=True
+        )
+
         self.url = urlunparse(url_parts)
 
     def encode_url_key(self, key):
@@ -624,7 +633,8 @@ class ReviewBoardServer(object):
                  api_token=None, agent=None, session=None, disable_proxy=False,
                  auth_callback=None, otp_token_callback=None,
                  verify_ssl=True, save_cookies=True, ext_auth_cookies=None,
-                 ca_certs=None, client_key=None, client_cert=None):
+                 ca_certs=None, client_key=None, client_cert=None,
+                 proxy_authorization=None):
         if not url.endswith('/'):
             url += '/'
 
@@ -731,12 +741,16 @@ class ReviewBoardServer(object):
         if agent:
             self.agent = agent
         else:
-            self.agent = ('RBTools/' + get_package_version()).encode('utf-8')
+            self.agent = 'RBTools/' + get_package_version()
 
         opener = build_opener(*handlers)
-        opener.addheaders = [
-            (str('User-agent'), str(self.agent)),
-        ]
+        headers = [(str('User-agent'), str(self.agent))]
+
+        if proxy_authorization:
+            headers.append((str('Proxy-Authorization'),
+                            str(proxy_authorization)))
+
+        opener.addheaders = headers
         install_opener(opener)
 
         self._cache = None
