@@ -37,10 +37,12 @@ class GitClient(SCMClient):
     """
 
     name = 'Git'
+    server_tool_names = 'Git,Perforce,Subversion'
 
     supports_commit_history = True
     supports_diff_exclude_patterns = True
     supports_no_renames = True
+    supports_parent_diffs = True
     supports_patch_revert = True
 
     can_amend_commit = True
@@ -89,7 +91,7 @@ class GitClient(SCMClient):
                                   none_on_ignored_error=True)
 
             if version_str:
-                m = re.search('(\d+)\.(\d+)\.(\d+)', version_str)
+                m = re.search(r'(\d+)\.(\d+)\.(\d+)', version_str)
 
                 if m:
                     git_version = (int(m.group(1)),
@@ -236,12 +238,12 @@ class GitClient(SCMClient):
 
         return result
 
-    def get_repository_info(self):
-        """Get repository information for the current Git working tree.
+    def get_local_path(self):
+        """Return the local path to the working tree.
 
         Returns:
-            rbtools.clients.RepositoryInfo:
-            The repository info structure.
+            unicode:
+            The filesystem path of the repository on the client system.
         """
         # Temporarily reset the toplevel. This is necessary for making things
         # work correctly in unit tests where we may be moving the cwd around a
@@ -260,9 +262,9 @@ class GitClient(SCMClient):
                               '--help": skipping Git')
                 return None
 
-        git_dir = self._get_git_dir()
+        self._git_dir = self._get_git_dir()
 
-        if git_dir is None:
+        if self._git_dir is None:
             return None
 
         # Sometimes core.bare is not set, and generates an error, so ignore
@@ -280,11 +282,25 @@ class GitClient(SCMClient):
             # Top level might not work on old git version se we use git dir
             # to find it.
             if (git_top.startswith(('fatal:', 'cygdrive')) or
-                not os.path.isdir(git_dir)):
-                git_top = git_dir
+                not os.path.isdir(self._git_dir)):
+                git_top = self._git_dir
 
             self._git_toplevel = os.path.abspath(git_top)
 
+        return self._git_toplevel
+
+    def get_repository_info(self):
+        """Return repository information for the current working tree.
+
+        Returns:
+            rbtools.clients.RepositoryInfo:
+            The repository info structure.
+        """
+        local_path = self.get_local_path()
+
+        if not local_path:
+            return None
+
         self._head_ref = self._execute(
             [self.git, 'symbolic-ref', '-q', 'HEAD'],
             ignore_errors=True).strip()
@@ -294,7 +310,7 @@ class GitClient(SCMClient):
         # directory. Otherwise, it may attempt to create one and scan
         # revisions, which can be slow. Also skip SVN detection if the git
         # repository was specified on command line.
-        git_svn_dir = os.path.join(git_dir, 'svn')
+        git_svn_dir = os.path.join(self._git_dir, 'svn')
 
         if (not getattr(self.options, 'repository_url', None) and
             os.path.isdir(git_svn_dir) and
@@ -326,8 +342,7 @@ class GitClient(SCMClient):
                         return SVNRepositoryInfo(path=path,
                                                  base_path=base_path,
                                                  local_path=local_path,
-                                                 uuid=uuid,
-                                                 supports_parent_diffs=True)
+                                                 uuid=uuid)
             else:
                 # Versions of git-svn before 1.5.4 don't (appear to) support
                 # 'git svn info'.  If we fail because of an older git install,
@@ -335,7 +350,7 @@ class GitClient(SCMClient):
                 # the user a hint about what to do next.
                 version = self._execute([self.git, 'svn', '--version'],
                                         ignore_errors=True)
-                version_parts = re.search('version (\d+)\.(\d+)\.(\d+)',
+                version_parts = re.search(r'version (\d+)\.(\d+)\.(\d+)',
                                           version)
                 svn_remote = self._execute(
                     [self.git, 'config', '--get', 'svn-remote.svn.url'],
@@ -350,7 +365,8 @@ class GitClient(SCMClient):
                                    'upgraded to version 1.5.4 or later.')
 
         # Okay, maybe Perforce (git-p4).
-        git_p4_ref = os.path.join(git_dir, 'refs', 'remotes', 'p4', 'master')
+        git_p4_ref = os.path.join(self._git_dir, 'refs', 'remotes', 'p4',
+                                  'master')
         if os.path.exists(git_p4_ref):
             data = self._execute([self.git, 'config', '--get', 'git-p4.port'],
                                  ignore_errors=True)
@@ -364,8 +380,7 @@ class GitClient(SCMClient):
                 self._type = self.TYPE_GIT_P4
                 return RepositoryInfo(path=port,
                                       base_path='',
-                                      local_path=self._git_toplevel,
-                                      supports_parent_diffs=True)
+                                      local_path=self._git_toplevel)
 
         # Nope, it's git then.
         # Check for a tracking branch and determine merge-base
@@ -385,13 +400,12 @@ class GitClient(SCMClient):
             # Central bare repositories don't have origin URLs.
             # We return git_dir instead and hope for the best.
             if not url:
-                url = os.path.abspath(git_dir)
+                url = os.path.abspath(self._git_dir)
 
         if url:
             return RepositoryInfo(path=url,
                                   base_path='',
-                                  local_path=self._git_toplevel,
-                                  supports_parent_diffs=True)
+                                  local_path=self._git_toplevel)
         return None
 
     def _get_git_dir(self):
@@ -460,33 +474,21 @@ class GitClient(SCMClient):
             unicode:
             The Review Board server URL, if available.
         """
-        # Scan first for dot files, since it's faster and will cover the
-        # user's $HOME/.reviewboardrc
-        server_url = super(GitClient, self).scan_for_server(repository_info)
+        if self._type == self.TYPE_GIT:
+            # TODO: Maybe support a server per remote later? Is that useful?
+            server_url = self._execute(
+                [self.git, 'config', '--get', 'reviewboard.url'],
+                ignore_errors=True).strip()
 
-        if server_url:
-            return server_url
-
-        # TODO: Maybe support a server per remote later? Is that useful?
-        url = self._execute([self.git, 'config', '--get', 'reviewboard.url'],
-                            ignore_errors=True).strip()
-        if url:
-            return url
-
-        if self._type == self.TYPE_GIT_SVN:
+            return server_url or None
+        elif self._type == self.TYPE_GIT_SVN:
             # Try using the reviewboard:url property on the SVN repo, if it
             # exists.
-            prop = SVNClient().scan_for_server_property(repository_info)
-
-            if prop:
-                return prop
+            return SVNClient().scan_for_server(repository_info)
         elif self._type == self.TYPE_GIT_P4:
-            prop = PerforceClient().scan_for_server(repository_info)
-
-            if prop:
-                return prop
-
-        return None
+            return PerforceClient().scan_for_server(repository_info)
+        else:
+            return None
 
     def get_raw_commit_message(self, revisions):
         """Extract the commit message based on the provided revision range.
@@ -554,7 +556,7 @@ class GitClient(SCMClient):
                                           ignore_errors=True).strip()
 
             if (defaultBranch and
-                os.path.exists(os.path.join(self._get_git_dir(), 'refs',
+                os.path.exists(os.path.join(self._git_dir, 'refs',
                                             'remotes', 'origin',
                                             defaultBranch))):
                 return 'origin/%s' % defaultBranch
