@@ -51,9 +51,11 @@ class MercurialClient(SCMClient):
     """
 
     name = 'Mercurial'
+    server_tool_names = 'Mercurial,Subversion'
 
     supports_commit_history = True
     supports_diff_exclude_patterns = True
+    supports_parent_diffs = True
     can_bookmark = True
     can_branch = True
     can_merge = True
@@ -80,7 +82,6 @@ class MercurialClient(SCMClient):
     # the actual character.
     _RECORD_SEP_ESC = r'\x1e'
 
-
     def __init__(self, executable='hg', **kwargs):
         """Initialize the client.
 
@@ -254,8 +255,21 @@ class MercurialClient(SCMClient):
 
         return history
 
+    def get_local_path(self):
+        """Return the local path to the working tree.
+
+        Returns:
+            unicode:
+            The filesystem path of the repository on the client system.
+        """
+        if not check_install([self._exe, '--help']):
+            logging.debug('Unable to execute "hg --help": skipping Mercurial')
+            return None
+
+        return self.hg_root
+
     def get_repository_info(self):
-        """Return the repository info object.
+        """Return repository information for the current working tree.
 
         Returns:
             rbtools.clients.RepositoryInfo:
@@ -283,8 +297,7 @@ class MercurialClient(SCMClient):
 
             return RepositoryInfo(path=path,
                                   base_path=base_path,
-                                  local_path=self.hg_root,
-                                  supports_parent_diffs=True)
+                                  local_path=self.hg_root)
 
     def parse_revision_spec(self, revisions=[]):
         """Parse the given revision spec.
@@ -320,16 +333,17 @@ class MercurialClient(SCMClient):
             ``commit_id`` (:py:class:`unicode`, optional):
                 The ID of the single commit being posted, if not using a range.
 
-            These will be used to generate the diffs to upload to Review Board (or
-            print). The diff for review will include the changes in (base, tip],
-            and the parent diff (if necessary) will include (parent, base].
+            These will be used to generate the diffs to upload to Review Board
+            (or print). The diff for review will include the changes in (base,
+            tip], and the parent diff (if necessary) will include (parent,
+            base].
 
-            If zero revisions are passed in, this will return the outgoing changes
-            from the parent of the working directory.
+            If zero revisions are passed in, this will return the outgoing
+            changes from the parent of the working directory.
 
-            If a single revision is passed in, this will return the parent of that
-            revision for "base" and the passed-in revision for "tip". This will
-            result in generating a diff for the changeset specified.
+            If a single revision is passed in, this will return the parent of
+            that revision for "base" and the passed-in revision for "tip". This
+            will result in generating a diff for the changeset specified.
 
             If two revisions are passed in, they will be used for the "base"
             and "tip" revisions, respectively.
@@ -517,8 +531,7 @@ class MercurialClient(SCMClient):
 
         return RepositoryInfo(path=root,
                               base_path=base_path,
-                              local_path=self.hg_root,
-                              supports_parent_diffs=True)
+                              local_path=self.hg_root)
 
     def _load_hgrc(self):
         """Load the hgrc file."""
@@ -1110,7 +1123,7 @@ class MercurialClient(SCMClient):
             parents = execute(
                 [self._exe, 'log', '-r', str(rev), '--template', '{parents}'],
                 env=self._hg_env)
-            parents = re.split(':[^\s]+\s*', parents)
+            parents = re.split(r':[^\s]+\s*', parents)
             parents = [int(p) for p in parents if p != '']
 
             parents = [p for p in parents if p not in outgoing_changesets]
@@ -1136,23 +1149,16 @@ class MercurialClient(SCMClient):
             unicode:
             The Review Board server URL, if available.
         """
-        # Scan first for dot files, since it's faster and will cover the
-        # user's $HOME/.reviewboardrc
-        server_url = \
-            super(MercurialClient, self).scan_for_server(repository_info)
+        server_url = self.hgrc.get('reviewboard.url', '').strip()
 
-        if not server_url and self.hgrc.get('reviewboard.url'):
-            server_url = self.hgrc.get('reviewboard.url').strip()
-
-        if not server_url and self._type == 'svn':
+        if server_url:
+            return server_url
+        elif self._type == 'svn':
             # Try using the reviewboard:url property on the SVN repo, if it
             # exists.
-            prop = SVNClient().scan_for_server_property(repository_info)
+            return SVNClient().scan_for_server(repository_info)
 
-            if prop:
-                return prop
-
-        return server_url
+        return None
 
     def _execute(self, cmd, *args, **kwargs):
         """Execute an hg command.
