@@ -5,11 +5,11 @@ from __future__ import unicode_literals
 import json
 import os
 import sys
+import unittest
 from functools import wraps
 from hashlib import md5
 
-from kgb import SpyAgency
-from nose import SkipTest
+import kgb
 from six.moves.urllib.request import urlopen
 
 from rbtools.api.client import RBClient
@@ -17,10 +17,11 @@ from rbtools.api.tests.base import MockResponse
 from rbtools.clients.errors import (InvalidRevisionSpecError,
                                     TooManyRevisionsError)
 from rbtools.clients.svn import SVNRepositoryInfo, SVNClient
-from rbtools.clients.tests import FOO1, FOO2, FOO3, SCMClientTests
+from rbtools.clients.tests import FOO1, FOO2, FOO3, SCMClientTestCase
 from rbtools.utils.checks import is_valid_version
 from rbtools.utils.filesystem import is_exe_in_path
 from rbtools.utils.process import execute
+from rbtools.utils.repository import get_repository_resource
 
 
 def svn_version_set_hash(svn16_hash, svn17_hash, svn19_hash):
@@ -50,7 +51,12 @@ def svn_version_set_hash(svn16_hash, svn17_hash, svn19_hash):
     return decorator
 
 
-class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
+_MATCH_URL_BASE = 'http://localhost:8080/api/repositories/'
+_MATCH_URL_TOOL = 'tool=Subversion'
+_MATCH_URL_FIELDS = 'only-fields=id%2Cname%2Cmirror_path%2Cpath'
+
+
+class SVNRepositoryMatchTests(kgb.SpyAgency, SCMClientTestCase):
     """Unit tests for rbtools.clients.svn.SVNRepositoryInfo."""
 
     payloads = {
@@ -71,7 +77,10 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
                 'stat': 'ok',
             },
         },
-        'http://localhost:8080/api/repositories/?tool=Subversion': {
+        (_MATCH_URL_BASE + '?' +
+            _MATCH_URL_FIELDS +
+            '&only-links=info&path=https%3A%2F%2Fsvn1.example.com%2F&' +
+            _MATCH_URL_TOOL): {
             'mimetype': 'application/vnd.reviewboard.org.repositories+json',
             'rsp': {
                 'repositories': [
@@ -83,7 +92,90 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
                         'path': 'https://svn1.example.com/',
                         'links': {
                             'info': {
-                                'href': ('https://localhost:8080/api/'
+                                'href': ('http://localhost:8080/api/'
+                                         'repositories/1/info/'),
+                                'method': 'GET',
+                            },
+                        },
+                    },
+                ],
+                'links': {
+                },
+                'total_results': 1,
+                'stat': 'ok',
+            },
+        },
+        (_MATCH_URL_BASE + '?' +
+            _MATCH_URL_FIELDS +
+            '&only-links=info&path=svn%2Bssh%3A%2F%2Fsvn2.example.com%2F&' +
+            _MATCH_URL_TOOL): {
+            'mimetype': 'application/vnd.reviewboard.org.repositories+json',
+            'rsp': {
+                'repositories': [
+                    {
+                        'id': 2,
+                        'name': 'SVN Repo 2',
+                        'path': 'https://svn2.example.com/',
+                        'mirror_path': 'svn+ssh://svn2.example.com/',
+                        'links': {
+                            'info': {
+                                'href': ('http://localhost:8080/api/'
+                                         'repositories/1/info/'),
+                                'method': 'GET',
+                            },
+                        },
+                    },
+                ],
+                'links': {
+                },
+                'total_results': 1,
+                'stat': 'ok',
+            },
+        },
+        (_MATCH_URL_BASE + '?' +
+            _MATCH_URL_FIELDS +
+            '&only-links=info&path=svn%2Bssh%3A%2F%2Fblargle%2F&' +
+            _MATCH_URL_TOOL): {
+            'mimetype': 'application/vnd.reviewboard.org.repositories+json',
+            'rsp': {
+                'repositories': [
+                ],
+                'links': {
+                },
+                'total_results': 0,
+                'stat': 'ok',
+            },
+        },
+        (_MATCH_URL_BASE + '?' +
+            _MATCH_URL_FIELDS +
+            '&only-links=&path=svn%2Bssh%3A%2F%2Fblargle%2F&' +
+            _MATCH_URL_TOOL): {
+            'mimetype': 'application/vnd.reviewboard.org.repositories+json',
+            'rsp': {
+                'repositories': [
+                ],
+                'links': {
+                },
+                'total_results': 0,
+                'stat': 'ok',
+            },
+        },
+        (_MATCH_URL_BASE + '?' +
+            _MATCH_URL_FIELDS +
+            '&only-links=info&' +
+            _MATCH_URL_TOOL): {
+            'mimetype': 'application/vnd.reviewboard.org.repositories+json',
+            'rsp': {
+                'repositories': [
+                    {
+                        # This one doesn't have a mirror_path, to emulate
+                        # Review Board 1.6.
+                        'id': 1,
+                        'name': 'SVN Repo 1',
+                        'path': 'https://svn1.example.com/',
+                        'links': {
+                            'info': {
+                                'href': ('http://localhost:8080/api/'
                                          'repositories/1/info/'),
                                 'method': 'GET',
                             },
@@ -96,7 +188,7 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
                         'mirror_path': 'svn+ssh://svn2.example.com/',
                         'links': {
                             'info': {
-                                'href': ('https://localhost:8080/api/'
+                                'href': ('http://localhost:8080/api/'
                                          'repositories/2/info/'),
                                 'method': 'GET',
                             },
@@ -105,8 +197,10 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
                 ],
                 'links': {
                     'next': {
-                        'href': ('http://localhost:8080/api/repositories/'
-                                 '?tool=Subversion&page=2'),
+                        'href': ('http://localhost:8080/api/repositories/?'
+                                 'only-links=info&tool=Subversion&'
+                                 'only-fields=id%2Cname%2Cmirror_path%2Cpath&'
+                                 'page=2'),
                         'method': 'GET',
                     },
                 },
@@ -114,7 +208,10 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
                 'stat': 'ok',
             },
         },
-        'http://localhost:8080/api/repositories/?tool=Subversion&page=2': {
+        (_MATCH_URL_BASE + '?' +
+            _MATCH_URL_FIELDS +
+            '&only-links=info&page=2&' +
+            _MATCH_URL_TOOL): {
             'mimetype': 'application/vnd.reviewboard.org.repositories+json',
             'rsp': {
                 'repositories': [
@@ -125,7 +222,7 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
                         'mirror_path': 'svn+ssh://svn3.example.com/',
                         'links': {
                             'info': {
-                                'href': ('https://localhost:8080/api/'
+                                'href': ('http://localhost:8080/api/'
                                          'repositories/3/info/'),
                                 'method': 'GET',
                             },
@@ -136,7 +233,7 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
                 'stat': 'ok',
             },
         },
-        'https://localhost:8080/api/repositories/1/info/': {
+        (_MATCH_URL_BASE + '1/info/'): {
             'mimetype': 'application/vnd.reviewboard.org.repository-info+json',
             'rsp': {
                 'info': {
@@ -147,7 +244,7 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
                 'stat': 'ok',
             },
         },
-        'https://localhost:8080/api/repositories/2/info/': {
+        (_MATCH_URL_BASE + '2/info/'): {
             'mimetype': 'application/vnd.reviewboard.org.repository-info+json',
             'rsp': {
                 'info': {
@@ -158,7 +255,7 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
                 'stat': 'ok',
             },
         },
-        'https://localhost:8080/api/repositories/3/info/': {
+        (_MATCH_URL_BASE + '3/info/'): {
             'mimetype': 'application/vnd.reviewboard.org.repository-info+json',
             'rsp': {
                 'info': {
@@ -172,14 +269,17 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
     }
 
     def setUp(self):
-        super(SVNRepositoryInfoTests, self).setUp()
+        super(SVNRepositoryMatchTests, self).setUp()
 
+        @self.spy_for(urlopen)
         def _urlopen(url, **kwargs):
             url = url.get_full_url()
 
             try:
                 payload = self.payloads[url]
             except KeyError:
+                print('Test requested unexpected URL "%s"' % url)
+
                 return MockResponse(404, {}, json.dumps({
                     'rsp': {
                         'stat': 'fail',
@@ -197,40 +297,54 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
                 },
                 json.dumps(payload['rsp']))
 
-        self.spy_on(urlopen, call_fake=_urlopen)
-
         self.api_client = RBClient('http://localhost:8080/')
         self.root_resource = self.api_client.get_root()
 
-    def test_find_server_repository_info_with_path_match(self):
-        """Testing SVNRepositoryInfo.find_server_repository_info with
-        path matching
+    def test_find_matching_server_repository_with_path_match(self):
+        """Testing SVNClient.find_matching_server_repository with path
+        match
         """
-        info = SVNRepositoryInfo('https://svn1.example.com/', '/', '')
+        url = 'https://svn1.example.com/'
+        self.options.repository_url = url
+        client = SVNClient(options=self.options)
 
-        repo_info = info.find_server_repository_info(self.root_resource)
-        self.assertEqual(repo_info, info)
-        self.assertEqual(repo_info.repository_id, 1)
+        repository, info = get_repository_resource(
+            self.root_resource,
+            tool=client,
+            repository_paths=url)
+        self.assertEqual(repository.id, 1)
 
-    def test_find_server_repository_info_with_mirror_path_match(self):
-        """Testing SVNRepositoryInfo.find_server_repository_info with
-        mirror_path matching
+    def test_find_matching_server_repository_with_mirror_path_match(self):
+        """Testing SVNClient.find_matching_server_repository with mirror_path
+        match
         """
-        info = SVNRepositoryInfo('svn+ssh://svn2.example.com/', '/', '')
+        url = 'svn+ssh://svn2.example.com/'
+        self.options.repository_url = url
+        client = SVNClient(options=self.options)
 
-        repo_info = info.find_server_repository_info(self.root_resource)
-        self.assertEqual(repo_info, info)
-        self.assertEqual(repo_info.repository_id, 2)
+        repository, info = get_repository_resource(
+            self.root_resource,
+            tool=client,
+            repository_paths=url)
+        self.assertEqual(repository.id, 2)
 
-    def test_find_server_repository_info_with_uuid_match(self):
-        """Testing SVNRepositoryInfo.find_server_repository_info with
-        UUID matching
+    def test_find_matching_server_repository_with_uuid_match(self):
+        """Testing SVNClient.find_matching_server_repository with UUID
+        match
         """
-        info = SVNRepositoryInfo('svn+ssh://blargle/', '/', 'UUID-3')
+        url = 'svn+ssh://blargle/'
+        self.options.repository_url = url
+        client = SVNClient(options=self.options)
 
-        repo_info = info.find_server_repository_info(self.root_resource)
-        self.assertNotEqual(repo_info, info)
-        self.assertEqual(repo_info.repository_id, 3)
+        self.spy_on(client.svn_info, op=kgb.SpyOpReturn({
+            'Repository UUID': 'UUID-3',
+        }))
+
+        repository, info = get_repository_resource(
+            self.root_resource,
+            tool=client,
+            repository_paths=url)
+        self.assertEqual(repository.id, 3)
 
     def test_relative_paths(self):
         """Testing SVNRepositoryInfo._get_relative_path"""
@@ -251,23 +365,48 @@ class SVNRepositoryInfoTests(SpyAgency, SCMClientTests):
             '/')
 
 
-class SVNClientTests(SCMClientTests):
+class SVNClientTests(SCMClientTestCase):
+    """Unit tests for SVNClient."""
+
+    @classmethod
+    def setup_checkout(cls, checkout_dir):
+        """Populate a Subversion checkout.
+
+        This will create a checkout of the sample Subversion repository stored
+        in the :file:`testdata` directory.
+
+        Args:
+            checkout_dir (unicode):
+                The top-level directory in which the checkout will be placed.
+
+        Returns:
+            The main checkout directory, or ``None`` if :command:`svn` isn't
+            in the path.
+        """
+        if not is_exe_in_path('svn'):
+            return None
+
+        cls.svn_dir = os.path.join(cls.testdata_dir, 'svn-repo')
+        cls.svn_repo_url = 'file://%s' % cls.svn_dir
+        cls.clone_dir = os.path.join(checkout_dir, 'svn-repo')
+
+        os.mkdir(checkout_dir, 0o700)
+        os.chdir(checkout_dir)
+        cls._run_svn(['co', cls.svn_repo_url, cls.clone_dir])
+
+        return cls.clone_dir
+
     def setUp(self):
+        if not is_exe_in_path('svn'):
+            raise unittest.SkipTest('svn not found in path')
+
         super(SVNClientTests, self).setUp()
 
-        if not is_exe_in_path('svn'):
-            raise SkipTest('svn not found in path')
-
-        self.svn_dir = os.path.join(self.testdata_dir, 'svn-repo')
-        self.clone_dir = self.chdir_tmp()
-        self.svn_repo_url = 'file://' + self.svn_dir
-        self._run_svn(['co', self.svn_repo_url, 'svn-repo'])
-        os.chdir(os.path.join(self.clone_dir, 'svn-repo'))
-
-        self.client = SVNClient(options=self.options)
         self.options.svn_show_copies_as_adds = None
+        self.client = SVNClient(options=self.options)
 
-    def _run_svn(self, command):
+    @classmethod
+    def _run_svn(cls, command):
         return execute(['svn'] + command, env=None, split_lines=False,
                        ignore_errors=False, extra_ignore_errors=())
 
@@ -593,8 +732,8 @@ class SVNClientTests(SCMClientTests):
         # Ensure valid SVN client version.
         if not is_valid_version(self.client.subversion_client_version,
                                 self.client.SHOW_COPIES_AS_ADDS_MIN_VERSION):
-            raise SkipTest('Subversion client is too old to test '
-                           '--show-copies-as-adds.')
+            raise unittest.SkipTest('Subversion client is too old to test '
+                                    '--show-copies-as-adds.')
 
         self.options.svn_show_copies_as_adds = state
 
@@ -643,8 +782,8 @@ class SVNClientTests(SCMClientTests):
         # Ensure valid SVN client version.
         if not is_valid_version(self.client.subversion_client_version,
                                 self.client.SHOW_COPIES_AS_ADDS_MIN_VERSION):
-            raise SkipTest('Subversion client is too old to test '
-                           'history_scheduled_with_commit().')
+            raise unittest.SkipTest('Subversion client is too old to test '
+                                    'history_scheduled_with_commit().')
 
         self._svn_add_dir('dir1')
         self._svn_add_dir('dir2')
@@ -684,8 +823,8 @@ class SVNClientTests(SCMClientTests):
         # Ensure valid SVN client version.
         if not is_valid_version(self.client.subversion_client_version,
                                 self.client.SHOW_COPIES_AS_ADDS_MIN_VERSION):
-            raise SkipTest('Subversion client is too old to test '
-                           'history_scheduled_with_commit().')
+            raise unittest.SkipTest('Subversion client is too old to test '
+                                    'history_scheduled_with_commit().')
 
         # While within a working copy which contains a scheduled commit with
         # addition-with-history, ensure history_scheduled_with_commit() is not
@@ -715,8 +854,8 @@ class SVNClientTests(SCMClientTests):
         # Ensure valid SVN client version.
         if not is_valid_version(self.client.subversion_client_version,
                                 self.client.SHOW_COPIES_AS_ADDS_MIN_VERSION):
-            raise SkipTest('Subversion client is too old to test '
-                           'history_scheduled_with_commit().')
+            raise unittest.SkipTest('Subversion client is too old to test '
+                                    'history_scheduled_with_commit().')
 
         # Lone file with history is also excluded.  In this case there should
         # be no SystemExit raised and an (empty) diff should be produced. Test
