@@ -1,7 +1,9 @@
-from __future__ import print_function, unicode_literals
+from __future__ import unicode_literals
 
 import argparse
 import inspect
+import io
+import json
 import logging
 import platform
 import os
@@ -19,10 +21,12 @@ from rbtools.api.errors import APIError, ServerInterfaceError
 from rbtools.api.transport.sync import SyncTransport
 from rbtools.clients import scan_usable_client
 from rbtools.clients.errors import OptionsCheckError
+from rbtools.deprecation import RemovedInRBTools40Warning
 from rbtools.utils.console import get_input, get_pass
 from rbtools.utils.filesystem import (cleanup_tempfiles, get_home_path,
                                       is_exe_in_path, load_config)
 from rbtools.utils.process import log_command_line
+from rbtools.utils.repository import get_repository_resource
 
 
 RB_MAIN = 'rbt'
@@ -42,6 +46,88 @@ class ParseError(CommandError):
     pass
 
 
+class JSONOutput(object):
+    """Output wrapper for JSON output.
+
+    JSON outputter class that stores Command outputs in python dictionary
+    and outputs as JSON object to output stream object. Commands should add any
+    structured output to this object. JSON output is then enabled with the
+    --json argument.
+
+    Version Added:
+        3.0
+    """
+    def __init__(self, output_stream):
+        """Initialize JSONOutput class.
+
+        Args:
+            output_stream (Object):
+                Object to output JSON object to.
+        """
+        self._output = {}
+        self._output_stream = output_stream
+
+    def add(self, key, value):
+        """Add a new key value pair.
+
+        Args:
+            key (unicode):
+                The key associated with the value to be added to dictionary.
+
+            value (object):
+                The value to attach to the key in the dictionary.
+        """
+        self._output[key] = value
+
+    def append(self, key, value):
+        """Add new value to an existing list associated with key.
+
+        Args:
+            key (unicode):
+                The key associated with the list to append to.
+
+            value (object):
+                The value to append to the list associated with key.
+
+        Raises:
+            KeyError:
+                The key was not found in the state.
+
+            AttributeError:
+                The existing value was not a list.
+        """
+        self._output[key].append(value)
+
+    def add_error(self, error):
+        """Add new error to 'errors' key.
+
+        Append a new error to the ``errors`` key, creating one if needed.
+
+        Args:
+            error (unicode):
+                The error that will be added to ``errors``.
+        """
+        self._output.setdefault('errors', []).append(error)
+
+    def add_warning(self, warning):
+        """Add new warning to 'warnings' key.
+
+        Append a new warning to the ``warnings`` key, creating one if needed.
+
+        Args:
+            warning (unicode):
+                The warning that will be added to ``warnings``.
+        """
+        self._output.setdefault('warnings', []).append(warning)
+
+    def print_to_stream(self):
+        """Output JSON string representation to output stream."""
+        self._output_stream.write(json.dumps(self._output,
+                                             indent=4,
+                                             sort_keys=True))
+        self._output_stream.write('\n')
+
+
 class SmartHelpFormatter(argparse.HelpFormatter):
     """Smartly formats help text, preserving paragraphs."""
 
@@ -61,6 +147,57 @@ class SmartHelpFormatter(argparse.HelpFormatter):
         return lines[:-1]
 
 
+class OutputWrapper(object):
+    """Wrapper for output of a command.
+
+    Wrapper around some output object that handles outputting messages.
+    Child classes specify the default object. The wrapper can handle
+    messages in either unicode or bytes.
+
+    Version Added:
+        3.0
+    """
+
+    def __init__(self, output_stream):
+        """Initialize with an output object to stream to.
+
+        Args:
+            output_stream (object):
+                The output stream to send command output to.
+        """
+        self.output_stream = output_stream
+
+    def write(self, msg, end='\n'):
+        """Write a message to the output stream.
+
+        Write a string to output stream object if defined, otherwise
+        do nothing. end specifies a string that should be appended to
+        the end of msg before being given to the output stream.
+
+        Args:
+            msg (unicode):
+                String to write to output stream.
+
+            end (unicode, optional):
+                String to append to end of msg. This defaults to ``\n```.
+        """
+        if self.output_stream:
+            if end:
+                if (isinstance(msg, bytes) and
+                    isinstance(end, six.string_types)):
+                    msg += end.encode('utf-8')
+                else:
+                    msg += end
+
+            self.output_stream.write(msg)
+
+    def new_line(self):
+        """Pass a new line character to output stream object.
+        """
+        if self.output_stream:
+            self.output_stream.write('\n')
+
+
 class Option(object):
     """Represents an option for a command.
 
@@ -126,8 +263,26 @@ class OptionGroup(object):
         self.option_list = option_list
 
     def add_to(self, parser, config={}, argv=[]):
-        """Adds the group and all its contained options to the parser."""
-        group = parser.add_argument_group(self.name, self.description)
+        """Add the group and all its contained options to the parser.
+
+        Args:
+            parser (argparse.ArgumentParser):
+                The command-line parser.
+
+            config (dict):
+                The loaded RBTools configuration.
+
+            argv (list, deprecated):
+                Unused legacy argument.
+        """
+        # First look for an existing group with the same name. This allows
+        # a BaseSubCommand to merge option groups with its parent
+        # BaseMultiCommand.
+        for group in parser._action_groups:
+            if group.title == self.name:
+                break
+        else:
+            group = parser.add_argument_group(self.name, self.description)
 
         for option in self.option_list:
             option.add_to(group, config, argv)
@@ -154,25 +309,120 @@ class Command(object):
     This class will handle retrieving the configuration, and parsing
     command line options.
 
-    ``description`` is a string containing a short description of the
-    command which is suitable for display in usage text.
-
     ``usage`` is a list of usage strings each showing a use case. These
     should not include the main rbt command or the command name; they
     will be added automatically.
 
-    ``args`` is a string containing the usage text for what arguments the
-    command takes.
+    Attributes:
+        api_client (rbtools.api.client.RBClient):
+            The API client. This will be ``None`` if :py:attr:`needs_api` is
+            ``False``.
 
-    ``option_list`` is a list of command line options for the command.
-    Each list entry should be an Option or OptionGroup instance.
+        api_root (rbtools.api.resource.RootResource):
+            The API root resource. This will be ``None`` if
+            :py:attr:`needs_api` is ``False``.
+
+        capabilities (rbtools.api.capabilities.Capabilities):
+            The API capabilities object. This will be ``None`` if
+            :py:attr:`needs_api` is ``False``.
+
+        repository (rbtools.api.resource.ItemResource):
+            The API resource corresponding to the repository.
+
+        repository_info (rbtools.clients.RepositoryInfo):
+            The repository info object. This will be ``None`` if
+            :py:attr:`needs_scm_client` is ``False``.
+
+        server_url (unicode):
+            The URL to the Review Board server.
+
+        stdout (OutputWrapper):
+            Standard unicode output wrapper that subclasses must write to.
+
+        stderr (OutputWrapper):
+            Standard unicode error output wrapper that subclasses must write
+            to.
+
+        stdout_bytes (OutputWrapper):
+            Standard byte output wrapper that subclasses must write to.
+
+        stderr_bytes (OutputWrapper):
+            Standard byte error output wrapper that subclasses must write to.
+
+        tool (rbtools.clients.SCMClient):
+            The SCM client. This will be ``None`` if
+            :py:attr:`needs_scm_client` is ``False``.
     """
 
+    #: The name of the command.
+    #:
+    #: Type:
+    #:     unicode
     name = ''
+
+    #: The author of the command.
+    #:
+    #: Type:
+    #:     unicode
     author = ''
+
+    #: A short description of the command, suitable for display in usage text.
+    #:
+    #: Type:
+    #:     unicode
     description = ''
+
+    #: Whether the command needs the API client.
+    #:
+    #: If this is set, the initialization of the command will set
+    #: :py:attr:`api_client` and :py:attr:`api_root`.
+    #:
+    #: Version Added:
+    #:     3.0
+    #:
+    #: Type:
+    #:     bool
+    needs_api = False
+
+    #: Whether the command needs the SCM client.
+    #:
+    #: If this is set, the initialization of the command will set
+    #: :py:attr:`repository_info` and :py:attr:`tool`.
+    #:
+    #: Version Added:
+    #:     3.0
+    #:
+    #: Type:
+    #:     bool
+    needs_scm_client = False
+
+    #: Whether the command needs the remote repository object.
+    #:
+    #: If this is set, the initialization of the command will set
+    #: :py:attr:`repository`.
+    #:
+    #: Version Added:
+    #:     3.0
+    #:
+    #: Type:
+    #:     bool
+    needs_repository = False
+
+    #: Usage text for what arguments the command takes.
+    #:
+    #: Arguments for the command are anything passed in other than defined
+    #: options (for example, revisions passed to :command:`rbt post`).
+    #:
+    #: Type:
+    #:     unicode
     args = ''
+
+    #: Command-line options for this command.
+    #:
+    #: Type:
+    #:     list of Option or OptionGroup
     option_list = []
+
     _global_options = [
         Option('-d', '--debug',
                action='store_true',
@@ -182,6 +432,13 @@ class Command(object):
                help='Displays debug output.',
                extended_help='This information can be valuable when debugging '
                              'problems running the command.'),
+        Option('--json',
+               action='store_true',
+               dest='json_output',
+               config_key='JSON_OUTPUT',
+               default=False,
+               added_in='3.0',
+               help='Output results as JSON data instead of text.')
     ]
 
     server_options = OptionGroup(
@@ -292,6 +549,13 @@ class Command(object):
                    config_key='CLIENT_CERT',
                    default=None,
                    help='Certificate for TLS client authentication.'),
+            Option('--proxy-authorization',
+                   dest='proxy_authorization',
+                   metavar='PROXY_AUTHORIZATION',
+                   config_key='PROXY_AUTHORIZATION',
+                   default=None,
+                   help='Value of the Proxy-Authorization header to send with '
+                        'HTTP requests.'),
         ]
     )
 
@@ -583,15 +847,36 @@ class Command(object):
                 this uses the transport defined in
                 :py:attr:`default_transport_cls`.
         """
+
         self.log = logging.getLogger('rb.%s' % self.name)
         self.transport_cls = transport_cls or self.default_transport_cls
+        self.api_client = None
+        self.api_root = None
+        self.capabilities = None
+        self.repository = None
+        self.repository_info = None
+        self.server_url = None
+        self.tool = None
+
+        self.stdout = OutputWrapper(sys.stdout)
+        self.stderr = OutputWrapper(sys.stderr)
+
+        if isinstance(sys.stdout, io.TextIOWrapper):
+            self.stderr_bytes = OutputWrapper(sys.stderr.buffer)
+            self.stdout_bytes = OutputWrapper(sys.stdout.buffer)
+        else:
+            # Python 2.x, or while we're running unit tests (where stdout and
+            # stderr are actually io.StringIO)
+            self.stderr_bytes = OutputWrapper(sys.stderr)
+            self.stdout_bytes = OutputWrapper(sys.stdout)
+
+        self.json = JSONOutput(sys.stdout)
 
     def create_parser(self, config, argv=[]):
         """Create and return the argument parser for this command."""
         parser = argparse.ArgumentParser(
             prog=RB_MAIN,
             usage=self.usage(),
-            add_help=False,
             formatter_class=SmartHelpFormatter)
 
         for option in self.option_list:
@@ -652,70 +937,57 @@ class Command(object):
 
         return logging.Formatter(fmt.format(color=color, reset=reset))
 
-    def init_logging(self):
-        """Initializes logging for the command.
+    def initialize(self):
+        """Initialize the command.
 
-        This will set up different log handlers based on the formatting we want
-        for the given levels.
-
-        The INFO log handler will just show the text, like a print statement.
-
-        WARNING and higher will show the level name as a prefix, in the form of
-        "LEVEL: message".
-
-        If debugging is enabled, a debug log handler will be set up showing
-        debug messages in the form of ">>> message", making it easier to
-        distinguish between debugging and other messages.
+        This will set up various prerequisites for commands. Individual command
+        subclasses can control what gets done by setting the various
+        ``needs_*`` attributes (as documented in this class).
         """
-        if sys.stdout.isatty():
-            # We only use colorized logging when writing to TTYs, so we don't
-            # bother initializing it then.
-            colorama.init()
+        if self.needs_repository:
+            # If we need the repository, we implicitly need the API and SCM
+            # client as well.
+            self.needs_api = True
+            self.needs_scm_client = True
 
-        root = logging.getLogger()
+        if self.needs_api:
+            self.server_url = self._init_server_url()
+            self.api_client, self.api_root = self.get_api(self.server_url)
+            self.capabilities = self.get_capabilities(self.api_root)
 
-        if self.options.debug:
-            handler = logging.StreamHandler()
-            handler.setFormatter(self._create_formatter(
-                'DEBUG', '{color}>>>{reset} %(message)s'))
-            handler.setLevel(logging.DEBUG)
-            handler.addFilter(LogLevelFilter(logging.DEBUG))
-            root.addHandler(handler)
+        if self.needs_scm_client:
+            # _init_server_url might have already done this, in the case that
+            # it needed to use the SCM client to detect the server name. Only
+            # repeat if necessary.
+            if self.repository_info is None and self.tool is None:
+                self.repository_info, self.tool = self.initialize_scm_tool(
+                    client_name=self.options.repository_type)
 
-            root.setLevel(logging.DEBUG)
-        else:
-            root.setLevel(logging.INFO)
+            # Some SCMs allow configuring the repository name in the SCM
+            # metadata. This is a legacy configuration, and is only used as a
+            # fallback for when the repository name is not specified through
+            # the config or command line.
+            if self.options.repository_name is None:
+                self.options.repository_name = \
+                    self.tool.get_repository_name()
 
-        # Handler for info messages. We'll treat these like prints.
-        handler = logging.StreamHandler()
-        handler.setFormatter(self._create_formatter(
-            'INFO', '{color}%(message)s{reset}'))
+            self.tool.capabilities = self.capabilities
 
-        handler.setLevel(logging.INFO)
-        handler.addFilter(LogLevelFilter(logging.INFO))
-        root.addHandler(handler)
+        if self.needs_repository:
+            self.repository, info = get_repository_resource(
+                api_root=self.api_root,
+                tool=self.tool,
+                repository_name=self.options.repository_name,
+                repository_paths=self.repository_info.path)
 
-        # Handlers for warnings, errors, and criticals. They'll show the
-        # level prefix and the message.
-        levels = (
-            ('WARNING', logging.WARNING),
-            ('ERROR', logging.ERROR),
-            ('CRITICAL', logging.CRITICAL),
-        )
+            if self.repository:
+                self.repository_info.update_from_remote(self.repository, info)
 
-        for level_name, level in levels:
-            handler = logging.StreamHandler()
-            handler.setFormatter(self._create_formatter(
-                level_name, '{color}%(levelname)s:{reset} %(message)s'))
-            handler.addFilter(LogLevelFilter(level))
-            handler.setLevel(level)
-            root.addHandler(handler)
-
-        logging.debug('RBTools %s', get_version_string())
-        logging.debug('Python %s', sys.version)
-        logging.debug('Running on %s', platform.platform())
-        logging.debug('Home = %s', get_home_path())
-        logging.debug('Current directory = %s', os.getcwd())
+        if self.options.json_output:
+            self.stdout.output_stream = None
+            self.stderr.output_stream = None
+            self.stderr_bytes.output_stream = None
+            self.stdout_bytes.output_stream = None
 
     def create_arg_parser(self, argv):
         """Create and return the argument parser.
@@ -763,7 +1035,8 @@ class Command(object):
             parser.error('Invalid number of arguments provided')
             sys.exit(1)
 
-        self.init_logging()
+        self._init_logging()
+        self.initialize()
         log_command_line('Command line: %s', argv)
 
         try:
@@ -775,6 +1048,7 @@ class Command(object):
                 raise
 
             logging.error(e)
+            self.json.add_error(str(e))
             exit_code = 1
         except CommandExit as e:
             exit_code = e.exit_code
@@ -786,10 +1060,21 @@ class Command(object):
             if self.options.debug:
                 raise
 
+            self.json.add_error('Internal error: %s: %s'
+                                % (type(e).__name__, e))
             logging.critical(e)
             exit_code = 1
 
         cleanup_tempfiles()
+
+        if self.options.json_output:
+            if 'errors' in self.json._output:
+                self.json.add('status', 'failed')
+            else:
+                self.json.add('status', 'success')
+
+            self.json.print_to_stream()
+
         sys.exit(exit_code)
 
     def initialize_scm_tool(self, client_name=None,
@@ -813,11 +1098,17 @@ class Command(object):
             A 2-tuple, containing the repository info structure and the tool
             instance.
         """
+        if not require_repository_info:
+            RemovedInRBTools40Warning.warn(
+                'The require_repository_info parameter to '
+                'Command.initialize_scm_tool is deprecated and will be '
+                'removed in RBTools 4.0. Commands which need to use only the '
+                'API should set the needs_api attribute.')
+
         repository_info, tool = scan_usable_client(
             self.config,
             self.options,
-            client_name=client_name,
-            require_repository_info=require_repository_info)
+            client_name=client_name)
 
         try:
             tool.check_options()
@@ -832,6 +1123,11 @@ class Command(object):
         If api_root is not provided we'll assume we want to
         initialize the tool using only local information
         """
+        RemovedInRBTools40Warning.warn(
+            'The Command.setup_tool method is deprecated and will be removed '
+            'in RBTools 4.0. Commands which need to use both the API and SCM '
+            'client should instead set the needs_api and needs_scm_client '
+            'attributes.')
         tool.capabilities = self.get_capabilities(api_root)
 
     def get_server_url(self, repository_info, tool):
@@ -848,18 +1144,15 @@ class Command(object):
             unicode:
             The server URL.
         """
-        if self.options.server:
-            server_url = self.options.server
-        elif tool and repository_info is not None:
-            server_url = tool.scan_for_server(repository_info)
-        else:
-            server_url = None
+        RemovedInRBTools40Warning.warn(
+            'The Command.get_server_url method is deprecated and will be '
+            'removed in RBTools 4.0. Commands which need the API client '
+            'should instead set the needs_api attribute.')
 
-        if not server_url:
-            raise CommandError('Unable to find a Review Board server for this '
-                               'source code tree.')
+        if self.server_url is None:
+            self.server_url = self._init_server_url()
 
-        return server_url
+        return self.server_url
 
     def credentials_prompt(self, realm, uri, username=None, password=None,
                            *args, **kwargs):
@@ -891,9 +1184,10 @@ class Command(object):
 
                 raise CommandError('Unable to log in to Review Board.')
 
-            print()
-            print('Please log in to the Review Board server at %s.' %
-                  urlparse(uri)[1])
+            self.stdout.new_line()
+            self.stdout.write('Please log in to the Review Board server at '
+                              '%s.'
+                              % urlparse(uri)[1])
 
             if username is None:
                 username = get_input('Username: ')
@@ -915,22 +1209,23 @@ class Command(object):
                                'required, but cannot be used with '
                                '--diff-filename=-')
 
-        print()
-        print('Please enter your two-factor authentication token for Review '
-              'Board.')
+        self.stdout.new_line()
+        self.stdout.write('Please enter your two-factor authentication '
+                          'token for Review Board.')
 
         if token_method == 'sms':
-            print('You should be getting a text message with '
-                  'an authentication token.')
-            print('Enter the token below.')
+            self.stdout.write('You should be getting a text message with '
+                              'an authentication token.')
+            self.stdout.write('Enter the token below.')
         elif token_method == 'call':
-            print('You should be getting an automated phone call with '
-                  'an authentication token.')
-            print('Enter the token below.')
+            self.stdout.write('You should be getting an automated phone '
+                              'call with an authentication token.')
+            self.stdout.write('Enter the token below.')
         elif token_method == 'generator':
-            print('Enter the token shown on your token generator app below.')
+            self.stdout.write('Enter the token shown on your token '
+                              'generator app below.')
 
-        print()
+        self.stdout.new_line()
 
         return get_pass('Token: ', require=True)
 
@@ -957,6 +1252,7 @@ class Command(object):
             ca_certs=self.options.ca_certs,
             client_key=self.options.client_key,
             client_cert=self.options.client_cert,
+            proxy_authorization=self.options.proxy_authorization,
             transport_cls=self.transport_cls)
 
     def get_api(self, server_url):
@@ -1023,6 +1319,319 @@ class Command(object):
         """
         raise NotImplementedError()
 
+    def _init_logging(self):
+        """Initializes logging for the command.
+
+        This will set up different log handlers based on the formatting we want
+        for the given levels.
+
+        The INFO log handler will just show the text, like a print statement.
+
+        WARNING and higher will show the level name as a prefix, in the form of
+        "LEVEL: message".
+
+        If debugging is enabled, a debug log handler will be set up showing
+        debug messages in the form of ">>> message", making it easier to
+        distinguish between debugging and other messages.
+        """
+        if sys.stdout.isatty():
+            # We only use colorized logging when writing to TTYs, so we don't
+            # bother initializing it then.
+            colorama.init()
+
+        root = logging.getLogger()
+
+        if self.options.debug:
+            handler = logging.StreamHandler()
+            handler.setFormatter(self._create_formatter(
+                'DEBUG', '{color}>>>{reset} %(message)s'))
+            handler.setLevel(logging.DEBUG)
+            handler.addFilter(LogLevelFilter(logging.DEBUG))
+            root.addHandler(handler)
+
+            root.setLevel(logging.DEBUG)
+        else:
+            root.setLevel(logging.INFO)
+
+        # Handler for info messages. We'll treat these like prints.
+        handler = logging.StreamHandler()
+        handler.setFormatter(self._create_formatter(
+            'INFO', '{color}%(message)s{reset}'))
+
+        handler.setLevel(logging.INFO)
+        handler.addFilter(LogLevelFilter(logging.INFO))
+        root.addHandler(handler)
+
+        # Handlers for warnings, errors, and criticals. They'll show the
+        # level prefix and the message.
+        levels = (
+            ('WARNING', logging.WARNING),
+            ('ERROR', logging.ERROR),
+            ('CRITICAL', logging.CRITICAL),
+        )
+
+        for level_name, level in levels:
+            handler = logging.StreamHandler()
+            handler.setFormatter(self._create_formatter(
+                level_name, '{color}%(levelname)s:{reset} %(message)s'))
+            handler.addFilter(LogLevelFilter(level))
+            handler.setLevel(level)
+            root.addHandler(handler)
+
+        logging.debug('RBTools %s', get_version_string())
+        logging.debug('Python %s', sys.version)
+        logging.debug('Running on %s', platform.platform())
+        logging.debug('Home = %s', get_home_path())
+        logging.debug('Current directory = %s', os.getcwd())
+
+    def _init_server_url(self):
+        """Initialize the server URL.
+
+        This will discover the URL to the Review Board server (if possible),
+        and store it in :py:attr:`server_url`.
+
+        Returns:
+            unicode:
+            The Review Board server URL.
+
+        Raises:
+            CommandError:
+                The Review Board server could not be detected.
+        """
+        # First use anything directly provided on the command line or in the
+        # config file. This may be None or empty.
+        server_url = self.options.server
+
+        # Now try to use the SCM to discover the server name. Several SCMs have
+        # ways for the administrator to configure the Review Board server in
+        # the SCM metadata.
+        if not server_url:
+            self.repository_info, self.tool = self.initialize_scm_tool(
+                client_name=self.options.repository_type)
+
+            if self.repository_info is not None and self.tool is not None:
+                server_url = self.tool.scan_for_server(self.repository_info)
+
+        # Finally, fall back on the TREES config. Once upon a time, we
+        # suggested that users create a TREES dict in their config
+        # file that specified the REVIEWBOARD_URL for multiple repository
+        # paths. This was never properly documented, and has long outlived its
+        # usefulness. We're continuing to support it for now but emit a
+        # deprecation warning.
+        if not server_url:
+            if 'TREES' in self.config:
+                trees = self.config['TREES']
+                path = None
+
+                # Some repositories will return a list of paths.
+                if isinstance(self.repository_info.path, list):
+                    for path in self.repository_info.path:
+                        if path in trees:
+                            break
+                    else:
+                        path = None
+                elif self.repository_info.path in trees:
+                    path = self.repository_info.path
+
+                if path and 'REVIEWBOARD_URL' in trees[path]:
+                    RemovedInRBTools40Warning.warn(
+                        'The TREES configuration for specifying the '
+                        'REVIEWBOARD_URL is no longer supported and will be '
+                        'removed in RBTools 4.0. If you have multiple Review '
+                        'Board servers for different repositories, create '
+                        'individual .reviewboardrc files in each repository '
+                        'to define REVIEWBOARD_URL.')
+                    server_url = trees[path]['REVIEWBOARD_URL']
+
+        if not server_url:
+            raise CommandError('Unable to find a Review Board server for this '
+                               'source code tree.')
+
+        return server_url
+
+    def _get_text_type(self, markdown):
+        """Return the appropriate text type for a field.
+
+        Args:
+            markdown (bool):
+                Whether the field should be interpreted as Markdown-formatted
+                text.
+
+        Returns:
+            unicode:
+            The text type value to set for the field.
+        """
+        if markdown and self.capabilities.has_capability('text', 'markdown'):
+            return 'markdown'
+        else:
+            return 'plain'
+
+
+class BaseSubCommand(Command):
+    """Abstract base class for a subcommand."""
+
+    #: The subcommand's help text.
+    #:
+    #: Type:
+    #:     unicode
+    help_text = ''
+
+    def __init__(self, options, config, *args, **kwargs):
+        """Initialize the subcommand.
+
+        Args:
+            options (argparse.Namespace):
+                The parsed options.
+
+            config (dict):
+                The loaded RBTools configuration.
+
+            *args (list):
+                Positional arguments to pass to the Command class.
+
+            **kwargs (dict):
+                Keyword arguments to pass to the Command class.
+        """
+        super(BaseSubCommand, self).__init__(*args, **kwargs)
+        self.options = options
+        self.config = config
+
+
+class BaseMultiCommand(Command):
+    """Abstract base class for commands which offer subcommands.
+
+    Some commands (such as :command:`rbt review`) want to offer many
+    subcommands.
+
+    Version Added:
+        3.0
+    """
+
+    #: The available subcommands.
+    #:
+    #: This is a list of BaseSubCommand sub classes.
+    #:
+    #: Type:
+    #:     list
+    subcommands = {}
+
+    #: Options common to all subcommands.
+    #:
+    #: Type:
+    #:     list
+    common_subcommand_option_list = []
+
+    def usage(self, command_cls=None):
+        """Return a usage string for the command.
+
+        Args:
+            command_cls (type, optional):
+                The subcommand class to generate usage information for.
+
+        Returns:
+            unicode:
+            The usage string.
+        """
+        if command_cls is None:
+            subcommand = ' <subcommand>'
+            description = self.description
+        else:
+            subcommand = ''
+            description = command_cls.description
+
+        usage = '%%(prog)s%s [options] %s' % (subcommand, self.args)
+
+        if description:
+            return '%s\n\n%s' % (usage, description)
+        else:
+            return usage
+
+    def create_parser(self, config, argv=[]):
+        """Create and return the argument parser for this command.
+
+        Args:
+            config (dict):
+                The loaded RBTools config.
+
+            argv (list, optional):
+                The argument list.
+
+        Returns:
+            argparse.ArgumentParser:
+            The argument parser.
+        """
+        subcommand_parsers = {}
+
+        prog = '%s %s' % (RB_MAIN, self.name)
+
+        # Set up a parent parser containing the options that will be shared.
+        #
+        # Ideally the globals would also be available to the main command,
+        # but it ends up leading to arguments on the main command overruling
+        # those on the subcommand, which is a problem for --json. For now,
+        # we are only sharing on the subcommands.
+        common_parser = argparse.ArgumentParser(add_help=False)
+
+        for option in self.common_subcommand_option_list:
+            option.add_to(common_parser, config, argv)
+
+        for option in self._global_options:
+            option.add_to(common_parser, config, argv)
+
+        # Set up the parser for the main command.
+        parser = argparse.ArgumentParser(
+            prog=prog,
+            usage=self.usage(),
+            formatter_class=SmartHelpFormatter)
+
+        for option in self.option_list:
+            option.add_to(parser, config, argv)
+
+        # Set up the parsers for each subcommand.
+        subparsers = parser.add_subparsers(
+            description=(
+                'To get additional help for these commands, run: '
+                '%s <subcommand> --help' % prog))
+
+        for command_cls in self.subcommands:
+            subcommand_name = command_cls.name
+
+            subparser = subparsers.add_parser(
+                subcommand_name,
+                usage=self.usage(command_cls),
+                formatter_class=SmartHelpFormatter,
+                prog='%s %s' % (parser.prog, subcommand_name),
+                description=command_cls.description,
+                help=command_cls.help_text,
+                parents=[common_parser])
+
+            for option in command_cls.option_list:
+                option.add_to(subparser, config, argv)
+
+            subparser.set_defaults(command_cls=command_cls)
+            subcommand_parsers[subcommand_name] = subparser
+
+        self.subcommand_parsers = subcommand_parsers
+
+        return parser
+
+    def initialize(self):
+        """Initialize the command."""
+        super(BaseMultiCommand, self).initialize()
+
+        command = self.options.command_cls(options=self.options,
+                                           config=self.config,
+                                           transport_cls=self.transport_cls)
+        command.stdout = self.stdout
+        command.stderr = self.stderr
+        command.json = self.json
+        command.initialize()
+        self.subcommand = command
+
+    def main(self):
+        """Run the command."""
+        self.subcommand.main()
+
 
 def find_entry_point_for_command(command_name):
     """Return an entry point for the given rbtools command.
