@@ -1,4 +1,4 @@
-from __future__ import print_function, unicode_literals
+from __future__ import unicode_literals
 
 import logging
 
@@ -33,6 +33,11 @@ class Land(Command):
 
     name = 'land'
     author = 'The Review Board Project'
+
+    needs_api = True
+    needs_scm_client = True
+    needs_repository = True
+
     args = '[<branch-name>]'
     option_list = [
         Option('--dest',
@@ -219,16 +224,27 @@ class Land(Command):
                 Whether to simulate landing without actually changing the
                 repository.
         """
+        json_data = {
+            'review_request_id': review_request.id,
+            'review_request_url': review_request.absolute_url,
+            'destination_branch': destination_branch,
+            'source_branch': None,
+        }
+
         if source_branch:
             review_commit_message = extract_commit_message(review_request)
             author = review_request.get_submitter()
 
+            json_data['source_branch'] = source_branch
+
             if squash:
-                print('Squashing branch "%s" into "%s".'
-                      % (source_branch, destination_branch))
+                self.stdout.write('Squashing branch "%s" into "%s".'
+                                  % (source_branch, destination_branch))
+                json_data['type'] = 'squash'
             else:
-                print('Merging branch "%s" into "%s".'
-                      % (source_branch, destination_branch))
+                self.stdout.write('Merging branch "%s" into "%s".'
+                                  % (source_branch, destination_branch))
+                json_data['type'] = 'merge'
 
             if not dry_run:
                 try:
@@ -242,14 +258,50 @@ class Land(Command):
                 except MergeError as e:
                     raise CommandError(six.text_type(e))
         else:
-            print('Applying patch from review request %s.' % review_request.id)
+            self.stdout.write('Applying patch from review request %s.'
+                              % review_request.id)
+
+            json_data['type'] = 'patch'
 
             if not dry_run:
                 self.patch(review_request.id,
                            squash=squash)
 
-        print('Review request %s has landed on "%s".' %
-              (review_request.id, self.options.destination_branch))
+        self.stdout.write('Review request %s has landed on "%s".'
+                          % (review_request.id,
+                             self.options.destination_branch))
+        self.json.append('landed_review_requests', json_data)
+
+    def initialize(self):
+        """Initialize the command.
+
+        This overrides Command.initialize in order to handle full review
+        request URLs on the command line. In this case, we want to parse that
+        URL in order to pull the server name and review request ID out of it.
+
+        Raises:
+            rbtools.commands.CommandError:
+                A review request URL passed in as the review request ID could
+                not be parsed correctly or included a bad diff revision.
+        """
+        review_request_id = self.options.rid
+
+        if review_request_id and review_request_id.startswith('http'):
+            server_url, review_request_id, diff_revision = \
+                parse_review_request_url(review_request_id)
+
+            if diff_revision and '-' in diff_revision:
+                raise CommandError('Interdiff patches are not supported: %s.'
+                                   % diff_revision)
+
+            if review_request_id is None:
+                raise CommandError('The URL %s does not appear to be a '
+                                   'review request.')
+
+            self.options.server = server_url
+            self.options.rid = review_request_id
+
+        super(Land, self).initialize()
 
     def main(self, branch_name=None, *args):
         """Run the command."""
@@ -258,15 +310,6 @@ class Land(Command):
         if branch_name:
             self.cmd_args.insert(0, branch_name)
 
-        repository_info, self.tool = self.initialize_scm_tool(
-            client_name=self.options.repository_type)
-        server_url = self.get_server_url(repository_info, self.tool)
-        api_client, api_root = self.get_api(server_url)
-        self.setup_tool(self.tool, api_root=api_root)
-
-        # Check if repository info on reviewboard server match local ones.
-        repository_info = repository_info.find_server_repository_info(api_root)
-
         if not self.tool.can_merge:
             raise CommandError('This command does not support %s repositories.'
                                % self.tool.name)
@@ -288,23 +331,17 @@ class Land(Command):
         if self.options.rid:
             is_local = branch_name is not None
             review_request_id = self.options.rid
-
-            if review_request_id.startswith('http'):
-                (server_url,
-                 review_request_id,
-                 diff_revision) = parse_review_request_url(review_request_id)
         else:
             try:
                 review_request = guess_existing_review_request(
-                    repository_info=repository_info,
-                    repository_name=self.options.repository_name,
-                    api_root=api_root,
-                    api_client=api_client,
+                    api_root=self.api_root,
+                    api_client=self.api_client,
                     tool=self.tool,
                     revisions=get_revisions(self.tool, self.cmd_args),
                     guess_summary=False,
                     guess_description=False,
-                    is_fuzzy_match_func=self._ask_review_request_match)
+                    is_fuzzy_match_func=self._ask_review_request_match,
+                    repository_id=self.repository.id)
             except ValueError as e:
                 raise CommandError(six.text_type(e))
 
@@ -316,7 +353,7 @@ class Land(Command):
             is_local = True
 
         try:
-            review_request = api_root.get_review_request(
+            review_request = self.api_root.get_review_request(
                 review_request_id=review_request_id)
         except APIError as e:
             raise CommandError('Error getting review request %s: %s'
@@ -339,6 +376,13 @@ class Land(Command):
         land_error = self.can_land(review_request)
 
         if land_error is not None:
+            self.json.add('is_approved', False)
+            self.json.add('approval_failure', {
+                'review_request_id': review_request.id,
+                'review_request_url': review_request.absolute_url,
+                'message': land_error,
+            })
+
             raise CommandError('Cannot land review request %s: %s'
                                % (review_request_id, land_error))
 
@@ -350,6 +394,8 @@ class Land(Command):
             'squash': self.options.squash,
         }
 
+        self.json.add('landed_review_requests', [])
+
         if self.options.recursive:
             # The dependency graph shows us which review requests depend on
             # which other ones. What we are actually after is the order to land
@@ -361,13 +407,21 @@ class Land(Command):
             dependencies = toposort(dependency_graph)[1:]
 
             if dependencies:
-                print('Recursively landing dependencies of review request %s.'
-                      % review_request_id)
+                self.stdout.write('Recursively landing dependencies of '
+                                  'review request %s.'
+                                  % review_request_id)
 
                 for dependency in dependencies:
                     land_error = self.can_land(dependency)
 
                     if land_error is not None:
+                        self.json.add('is_approved', False)
+                        self.json.add('approval_failure', {
+                            'review_request_id': review_request.id,
+                            'review_request_url': review_request.absolute_url,
+                            'message': land_error,
+                        })
+
                         raise CommandError(
                             'Aborting recursive land of review request %s.\n'
                             'Review request %s cannot be landed: %s'
@@ -376,13 +430,15 @@ class Land(Command):
                 for dependency in reversed(dependencies):
                     self.land(review_request=dependency, **land_kwargs)
 
+        self.json.add('is_approved', True)
+
         self.land(review_request=review_request,
                   source_branch=branch_name,
                   **land_kwargs)
 
         if self.options.push:
-            print('Pushing branch "%s" upstream'
-                  % self.options.destination_branch)
+            self.stdout.write('Pushing branch "%s" upstream'
+                              % self.options.destination_branch)
 
             if not self.options.dry_run:
                 try:
