@@ -2,19 +2,20 @@
 
 from __future__ import unicode_literals
 
-import os.path
 import sys
 
+import kgb
+
 from rbtools import get_version_string
-from rbtools.commands import main as rbt_main
+from rbtools.commands import JSONOutput, main as rbt_main
+from rbtools.testing import TestCase
 from rbtools.utils.process import execute
-from rbtools.utils.testbase import RBTestBase
 
 
 _rbt_path = rbt_main.__file__
 
 
-class MainCommandTests(RBTestBase):
+class MainCommandTests(TestCase):
     """Tests for RBT help command and rbt command help options."""
 
     def test_help_command(self):
@@ -118,3 +119,65 @@ class MainCommandTests(RBTestBase):
             The resulting output from the command.
         """
         return execute([sys.executable, _rbt_path] + list(args))
+
+
+class JSONOutputTests(kgb.SpyAgency, TestCase):
+    """Tests for JSON output wrapper for --json command.
+    """
+
+    def setUp(self):
+        """Set up the test suite."""
+        super(JSONOutputTests, self).setUp()
+
+        self.json = JSONOutput(sys.stdout)
+
+    def test_init(self):
+        """Testing JSONOutput instantiates given stream object"""
+        self.assertIs(self.json._output_stream, sys.stdout)
+        self.assertEqual(len(self.json._output), 0)
+
+    def test_add(self):
+        """Testing JSONOutput.add adds a key value pair to output"""
+        self.json.add('key', 'value')
+
+        self.assertEqual(self.json._output['key'], 'value')
+
+    def test_append(self):
+        """Testing JSONOutput.append appends to list associated with a key"""
+        self.json.add('test', [])
+        self.json.append('test', 'test')
+
+        self.assertEqual(self.json._output['test'], ['test'])
+
+        try:
+            self.json.append('nonexistent', 'test')
+        except KeyError:
+            pass
+
+        self.assertNotIn('nonexistent', self.json._output)
+
+    def test_add_error_without_key(self):
+        """Testing JSONOutput.add_error without existing key"""
+        self.json.add_error('test_error')
+
+        self.assertEqual(self.json._output['errors'], ['test_error'])
+
+    def test_add_error_with_key(self):
+        """Testing JSONOutput.add_error with existing key"""
+        self.json.add_error('test_error')
+        self.json.add_error('test_error2')
+        self.assertEqual(self.json._output['errors'],
+                         ['test_error', 'test_error2'])
+
+    def test_add_warning_without_key(self):
+        """Testing JSONOutput.add_warning without existing key"""
+        self.json.add_warning('test_warning')
+
+        self.assertEqual(self.json._output['warnings'], ['test_warning'])
+
+    def test_add_warning_with_key(self):
+        """Testing JSONOutput.add_warning with existing key"""
+        self.json.add_warning('test_warning')
+        self.json.add_warning('test_warning2')
+        self.assertEqual(self.json._output['warnings'],
+                         ['test_warning', 'test_warning2'])
